@@ -37,7 +37,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 
 #include <nbdkit-plugin.h>
 
@@ -205,14 +204,25 @@ iso_open (int readonly)
 static int64_t
 iso_get_size (void *handle)
 {
-  struct stat statbuf;
+  int64_t r;
 
-  if (fstat (fd, &statbuf) == -1) {
-    nbdkit_error ("fstat: %m");
+  r = device_size (fd, NULL);
+  if (r == -1) {
+    nbdkit_error ("device_size: %m");
     return -1;
   }
+  return r;
+}
 
-  return statbuf.st_size;
+/* Make this look more like a real CD. */
+static int
+iso_block_size (void *handle,
+                uint32_t *minimum, uint32_t *preferred, uint32_t *maximum)
+{
+  *minimum = 1;
+  *preferred = 2048;
+  *maximum = 0xffffffff;
+  return 0;
 }
 
 /* Serves the same data over multiple connections. */
@@ -264,6 +274,7 @@ static struct nbdkit_plugin plugin = {
   .get_ready         = iso_get_ready,
   .open              = iso_open,
   .get_size          = iso_get_size,
+  .block_size        = iso_block_size,
   .can_multi_conn    = iso_can_multi_conn,
   .can_cache         = iso_can_cache,
   .pread             = iso_pread,
