@@ -624,6 +624,22 @@ vddk_dump_plugin (void)
   }
 #endif
 
+  /* List transport modes if available. */
+  if (library_version > 0) {
+    const char *modes;
+    VixError err;
+
+    err = VixDiskLib_InitEx (VDDK_MAJOR, VDDK_MINOR,
+                             NULL, NULL, NULL,
+                             libdir, config);
+    if (err == VIX_OK) {
+      modes = VixDiskLib_ListTransportModes ();
+      if (modes)
+        printf ("vddk_transport_modes=%s\n", modes);
+      VixDiskLib_Exit ();
+    }
+  }
+
   /* Note we print all VDDK APIs found here, not just the optional
    * ones.  That is so if we update the baseline VDDK in future and
    * make optional into required APIs, the output doesn't change.
@@ -873,19 +889,19 @@ vddk_get_size (void *handle)
 {
   struct vddk_handle *h = handle;
   VixDiskLibInfo *info;
-  int64_t size;
   struct command info_cmd = { .type = INFO, .ptr = &info };
 
   if (send_command_and_wait (h, &info_cmd) == -1)
     return -1;
 
-  size = info->capacity * (int64_t)VIXDISKLIB_SECTOR_SIZE;
+  /* Compute the size and cache it into the handle. */
+  h->size = info->capacity * VIXDISKLIB_SECTOR_SIZE;
 
   VDDK_CALL_START (VixDiskLib_FreeInfo, "info")
     VixDiskLib_FreeInfo (info);
   VDDK_CALL_END (VixDiskLib_FreeInfo, 0);
 
-  return size;
+  return h->size;
 }
 
 /* Advertise most efficient block sizes. */
