@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,12 +43,9 @@
 
 #include "internal.h"
 
-/* Note: preserves the previous value of errno. */
 void
-log_stderr_verror (const char *fs, va_list args)
+log_stderr_verror (int orig_errno, const char *fs, va_list args)
 {
-  int err = errno;              /* must be first line of function */
-
   const char *name = threadlocal_get_name ();
   size_t instance_num = threadlocal_get_instance_num ();
   int tty;
@@ -69,7 +66,7 @@ log_stderr_verror (const char *fs, va_list args)
   }
 
   fprintf (stderr, "error: ");
-  errno = err;                  /* must restore in case fs contains %m */
+  errno = orig_errno;        /* must restore in case fs contains %m */
   vfprintf (stderr, fs, args);
   fprintf (stderr, "\n");
 
@@ -78,6 +75,4 @@ log_stderr_verror (const char *fs, va_list args)
 #ifdef HAVE_FUNLOCKFILE
   funlockfile (stderr);
 #endif
-
-  errno = err;                  /* must be last line of function */
 }
