@@ -63,6 +63,7 @@
 
 #include <dlfcn.h>
 
+#include "ascii-ctype.h"
 #include "ascii-string.h"
 #include "exit-with-parent.h"
 #include "nbd-protocol.h"
@@ -80,7 +81,6 @@
 #endif
 
 static char *make_random_fifo (void);
-static char *make_uri (void);
 static struct backend *open_plugin_so (size_t i, const char *filename,
                                        int short_name);
 static struct backend *open_filter_so (struct backend *next, size_t i,
@@ -98,18 +98,22 @@ bool exit_with_parent;          /* --exit-with-parent */
 const char *export_name;        /* -e */
 bool foreground;                /* -f */
 const char *ipaddr;             /* -i */
+bool keepalive;                 /* --keepalive */
 enum log_to log_to = LOG_TO_DEFAULT; /* --log */
 unsigned mask_handshake = ~0U;  /* --mask-handshake */
 bool newstyle = true;           /* false = -o, true = -n */
+bool no_mc;                     /* --no-meta-contexts */
 bool no_sr;                     /* --no-sr */
 char *pidfile;                  /* -P */
 const char *port;               /* -p */
+bool print_uri;                 /* --print-uri */
 bool read_only;                 /* -r */
 const char *run;                /* --run */
 bool listen_stdin;              /* -s */
 const char *selinux_label;      /* --selinux-label */
 bool swap;                      /* --swap */
 unsigned threads;               /* -t */
+unsigned timeout_secs, timeout_nsecs; /* --timeout */
 int tls;                        /* --tls : 0=off 1=on 2=require */
 const char *tls_certificates_dir; /* --tls-certificates */
 const char *tls_psk;            /* --tls-psk */
@@ -120,7 +124,6 @@ bool verbose;                   /* -v */
 bool vsock;                     /* --vsock */
 
 enum service_mode service_mode; /* serving over TCP, Unix, etc */
-char *uri;                      /* NBD URI */
 bool configured;                /* .config_complete done */
 int saved_stdin = -1;           /* dup'd stdin during -s/--run */
 int saved_stdout = -1;          /* dup'd stdout during -s/--run */
@@ -144,11 +147,22 @@ static char *random_fifo = NULL;       /* Path to -U - socket */
 static void
 usage (void)
 {
+  extern const char *synopsis;
+
   /* --{short,long}-options remain undocumented */
-  const char *opt_list =
-#include "synopsis.c"
-	  ;
-  printf ("%s\n", opt_list);
+  printf ("%s\n", synopsis);
+
+  /* This blurb matches the DESCRIPTION section of the man page: */
+  printf (
+    "Network Block Device (NBD) is a network protocol for accessing block\n"
+    "devices over the network.  Block devices are hard disks and things that\n"
+    "behave like hard disks such as disk images and virtual machines.\n"
+    "\n"
+    "nbdkit is both a toolkit for creating NBD servers from “unconventional”\n"
+    "sources, and the name of an NBD server.  nbdkit ships with many plugins\n"
+    "for performing common tasks like serving local files.\n"
+    "\n"
+  );
   printf ("Please read the nbdkit(1) manual page for full usage.\n");
 }
 
@@ -210,6 +224,11 @@ dump_config (void)
 #endif
   printf ("%s=%s\n", "soext", SOEXT);
   printf ("%s=%s\n", "sysconfdir", sysconfdir);
+#ifdef HAVE_TIMEOUT_OPTION
+  printf ("timeout_option=yes\n");
+#else
+  printf ("timeout_option=no\n");
+#endif
 #ifdef HAVE_GNUTLS
   printf ("tls=yes\n");
 #else
@@ -220,6 +239,11 @@ dump_config (void)
     printf ("%s=%s\n", "version_extra", NBDKIT_VERSION_EXTRA);
   printf ("%s=%d\n", "version_major", NBDKIT_VERSION_MAJOR);
   printf ("%s=%d\n", "version_minor", NBDKIT_VERSION_MINOR);
+#if defined (AF_VSOCK) && defined (VMADDR_CID_ANY)
+  printf ("vsock_option=yes\n");
+#else
+  printf ("vsock_option=no\n");
+#endif
 #ifdef HAVE_LIBZSTD
   printf ("zstd=yes\n");
 #else
@@ -327,6 +351,14 @@ main (int argc, char *argv[])
       }
       break;
 
+    case HELP_OPTION:
+      help = true;
+      break;
+
+    case KEEPALIVE_OPTION:
+      keepalive = true;
+      break;
+
     case LOG_OPTION:
       if (strcmp (optarg, "stderr") == 0)
         log_to = LOG_TO_STDERR;
@@ -350,6 +382,24 @@ main (int argc, char *argv[])
       }
       exit (EXIT_SUCCESS);
 
+    case MASK_HANDSHAKE_OPTION:
+      if (nbdkit_parse_unsigned ("mask-handshake",
+                                 optarg, &mask_handshake) == -1)
+        exit (EXIT_FAILURE);
+      break;
+
+    case NO_MC_OPTION:
+      no_mc = true;
+      break;
+
+    case NO_SR_OPTION:
+      no_sr = true;
+      break;
+
+    case PRINT_URI:
+      print_uri = true;
+      break;
+
     case RUN_OPTION:
       if (socket_activation) {
         fprintf (stderr, "%s: cannot use socket activation with --run flag\n",
@@ -375,6 +425,18 @@ main (int argc, char *argv[])
       swap = 1;
       break;
 
+    case TIMEOUT_OPTION:
+#ifdef HAVE_TIMEOUT_OPTION
+      if (nbdkit_parse_delay ("timeout", optarg,
+                              &timeout_secs, &timeout_nsecs) == -1)
+        exit (EXIT_FAILURE);
+      break;
+#else
+      fprintf (stderr, "%s: --timeout is not supported on this platform\n",
+               program_name);
+      exit (EXIT_FAILURE);
+#endif
+
     case TLS_OPTION:
       tls_set_on_cli = true;
       if (ascii_strcasecmp (optarg, "require") == 0 ||
@@ -397,13 +459,7 @@ main (int argc, char *argv[])
       break;
 
     case TLS_VERIFY_PEER_OPTION:
-#ifdef HAVE_GNUTLS_SESSION_SET_VERIFY_CERT
       tls_verify_peer = true;
-#else
-      nbdkit_error ("--tls-verify-peer: "
-                    "GnuTLS >= 3.4.6 is required for this feature");
-      exit (EXIT_FAILURE);
-#endif
       break;
 
     case VSOCK_OPTION:
@@ -449,20 +505,10 @@ main (int argc, char *argv[])
       ipaddr = optarg;
       break;
 
-    case MASK_HANDSHAKE_OPTION:
-      if (nbdkit_parse_unsigned ("mask-handshake",
-                                 optarg, &mask_handshake) == -1)
-        exit (EXIT_FAILURE);
-      break;
-
     case 'n':
       newstyle = true;
       break;
 
-    case NO_SR_OPTION:
-      no_sr = true;
-      break;
-
     case 'o':
       newstyle = false;
       break;
@@ -537,10 +583,6 @@ main (int argc, char *argv[])
       version = true;
       break;
 
-    case HELP_OPTION:
-      help = true;
-      break;
-
     default:
       usage ();
       exit (EXIT_FAILURE);
@@ -660,11 +702,6 @@ main (int argc, char *argv[])
    * Note this may be NULL.
    */
   uri = make_uri ();
-  if (uri)
-    debug ("NBD URI: %s", uri);
-  else
-    debug ("no NBD URI because service mode is %s",
-           service_mode_string (service_mode));
 
   /* The remaining command line arguments are the plugin name and
    * parameters.  If --help, --version or --dump-plugin were specified
@@ -824,6 +861,22 @@ main (int argc, char *argv[])
    */
   top->get_ready (top);
 
+  /* Print the URI.  Do it just before we close stdio, especially
+   * before setting 'configured' to true which makes nbdkit_stdio_safe
+   * false.
+   */
+  if (print_uri && uri && nbdkit_stdio_safe ()) {
+    printf ("%s\n", uri);
+    printf ("Shell-quoted URI: ");
+    shell_quote (uri, stdout);
+    printf ("\n");
+    printf ("Command to query the NBD endpoint:\n");
+    printf ("  nbdinfo ");
+    shell_quote (uri, stdout);
+    printf ("\n");
+    fflush (stdout);
+  }
+
   switch_stdio ();
   configured = true;
 
@@ -906,81 +959,71 @@ service_mode_string (enum service_mode sm)
   }
 }
 
-static char *
-make_uri (void)
+/* Is it the name of a possible nbdkit-<name>-plugin or
+ * nbdkit-<name>-filter that might be a Fedora or Debian package?
+ * This matches more than all possible names, but it's just a guide
+ * for help output.
+ */
+static int
+is_possible_package_name (const char *name)
 {
-  FILE *fp;
-  size_t len = 0;
-  char *r = NULL;
+  size_t i, n = strlen (name);
 
-  switch (service_mode) {
-  case SERVICE_MODE_SOCKET_ACTIVATION:
-  case SERVICE_MODE_LISTEN_STDIN:
-    /* can't form a URI, uri will be NULL */
-    return NULL;
-  default: ;
+  for (i = 0; i < n; ++i) {
+    char c = name[i];
+    if (ascii_isalnum (c))
+      continue;
+    if (c == '-' || c == '_' || c == '.')
+      continue;
+    return 0;
   }
+  return 1;
+}
 
-  fp = open_memstream (&r, &len);
-  if (fp == NULL) {
-    perror ("open_memstream");
-    exit (EXIT_FAILURE);
+/* Common error handling for dlopen(3) failures in open_plugin_so or
+ * open_filter_so.
+ *
+ * The error message is held implicitly in dlerror(3).
+ *
+ * 'what' == "plugin" | "filter"
+ *
+ * 'filename' is the file we were trying to dlopen.
+ *
+ * 'short_name' is the plugin/filter short name that was passed on the
+ * nbdkit command line, or NULL if a short name was used (eg the
+ * plugin was opened using a full path).
+ */
+static void
+failed_to_load_error (const char *what,
+                      const char *filename,
+                      const char *short_name)
+{
+  fprintf (stderr, "%s: error: cannot open %s \"", program_name, what);
+  c_string_quote (filename, stderr);
+  fprintf (stderr, "\": ");
+  c_string_quote (dlerror (), stderr);
+  fprintf (stderr, "\n");
+  if (short_name && is_possible_package_name (short_name)) {
+    fprintf (stderr,
+             "\n"
+             "To add this functionality you might need to install a separate\n"
+             "%s package such as nbdkit-%s-%s (Fedora) or\n"
+             "nbdkit-%s-%s (Debian).\n",
+             what, short_name, what, what, short_name);
   }
-
-  switch (service_mode) {
-  case SERVICE_MODE_UNIXSOCKET:
-    fprintf (fp, "nbd%s+unix://", tls == 2 ? "s" : "");
-    if (export_name && strcmp (export_name, "") != 0) {
-      putc ('/', fp);
-      uri_quote (export_name, fp);
-    }
-    fprintf (fp, "?socket=");
-    uri_quote (unixsocket, fp);
-    break;
-  case SERVICE_MODE_VSOCK:
-    /* 1 = VMADDR_CID_LOCAL */
-    fprintf (fp, "nbd%s+vsock://1", tls == 2 ? "s" : "");
-    if (port) {
-      putc (':', fp);
-      fputs (port, fp);
-    }
-    if (export_name && strcmp (export_name, "") != 0) {
-      putc ('/', fp);
-      uri_quote (export_name, fp);
-    }
-    break;
-  case SERVICE_MODE_TCPIP:
-    fprintf (fp, "nbd%s://localhost", tls == 2 ? "s" : "");
-    if (port) {
-      putc (':', fp);
-      fputs (port, fp);
-    }
-    if (export_name && strcmp (export_name, "") != 0) {
-      putc ('/', fp);
-      uri_quote (export_name, fp);
-    }
-    break;
-
-  case SERVICE_MODE_SOCKET_ACTIVATION:
-  case SERVICE_MODE_LISTEN_STDIN:
-    /* these labels are not-reachable, see above */
-    /*FALLTHROUGH*/
-  default:
-    abort ();
-  }
-
-  if (fclose (fp) == EOF) {
-    perror ("memstream failed");
-    exit (EXIT_FAILURE);
-  }
-
-  return r;
+  fprintf (stderr,
+           "\n"
+           "Use '%s --help' or "
+           "read the nbdkit(1) manual page for documentation.\n",
+           program_name);
+  exit (EXIT_FAILURE);
 }
 
 static struct backend *
 open_plugin_so (size_t i, const char *name, int short_name)
 {
   struct backend *ret;
+  const char *orig_name = name;
   char *filename = (char *) name;
   bool free_filename = false;
   void *dl;
@@ -998,15 +1041,8 @@ open_plugin_so (size_t i, const char *name, int short_name)
   }
 
   dl = dlopen (filename, RTLD_NOW|RTLD_GLOBAL);
-  if (dl == NULL) {
-    fprintf (stderr,
-             "%s: error: cannot open plugin '%s': %s\n"
-             "Use '%s --help' or "
-             "read the nbdkit(1) manual page for documentation.\n",
-             program_name, name, dlerror (),
-             program_name);
-    exit (EXIT_FAILURE);
-  }
+  if (dl == NULL)
+    failed_to_load_error ("plugin", filename, short_name ? orig_name : NULL);
 
   /* Initialize the plugin.  See dlopen(3) to understand C weirdness. */
   dlerror ();
@@ -1034,6 +1070,7 @@ open_filter_so (struct backend *next, size_t i,
                 const char *name, int short_name)
 {
   struct backend *ret;
+  const char *orig_name = name;
   char *filename = (char *) name;
   bool free_filename = false;
   void *dl;
@@ -1051,11 +1088,8 @@ open_filter_so (struct backend *next, size_t i,
   }
 
   dl = dlopen (filename, RTLD_NOW|RTLD_GLOBAL);
-  if (dl == NULL) {
-    fprintf (stderr, "%s: error: cannot open filter '%s': %s\n",
-             program_name, name, dlerror ());
-    exit (EXIT_FAILURE);
-  }
+  if (dl == NULL)
+    failed_to_load_error ("filter", filename, short_name ? orig_name : NULL);
 
   /* Initialize the filter.  See dlopen(3) to understand C weirdness. */
   dlerror ();
@@ -1110,7 +1144,7 @@ start_serving (void)
      * the number of file descriptors from FIRST_SOCKET_ACTIVATION_FD
      * to FIRST_SOCKET_ACTIVATION_FD+socket_activation-1.
      */
-    if (sockets_reserve (&socks, socket_activation) == -1) {
+    if (sockets_reserve_exactly (&socks, socket_activation) == -1) {
       perror ("realloc");
       exit (EXIT_FAILURE);
     }
