@@ -43,7 +43,7 @@ do_test ()
 {
     vg=; [ "$NBDKIT_VALGRIND" = "1" ] && vg="-valgrind"
     case "$1$vg" in
-        python-valgrind | ruby-valgrind | tcl-valgrind)
+        python-valgrind | tcl-valgrind)
             echo "$0: skipping $1$vg because this language doesn't support valgrind"
             ;;
         example4*)
@@ -63,6 +63,13 @@ do_test ()
                 run_test $1
             fi
             ;;
+        gcs*)
+            # Requires Python plugin and google-cloud-storage library.
+            if nbdkit python --version && \
+                $PYTHON -c 'import google.cloud.storage'; then
+                run_test $1
+            fi
+            ;;
         *)
             run_test $1
             ;;
