@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -35,20 +35,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <string.h>
 #undef NDEBUG /* Keep test strong even for nbdkit built without assertions */
 #include <assert.h>
 
 #include <nbdkit-filter.h>
 
-#define THREAD_MODEL NBDKIT_THREAD_MODEL_PARALLEL
-
 #define str(s) #s
 #define DEBUG_FUNCTION nbdkit_debug ("%s: %s", layer, __func__)
 
-/* Perform sanity checking on next_ops(nxdata) stability */
+/* Perform sanity checking on nbdkit_next stability */
+static nbdkit_backend *saved_backend;
 struct handle {
-  void *nxdata;
-  struct nbdkit_next_ops *next_ops;
+  nbdkit_next *next;
 };
 
 static void
@@ -64,7 +63,7 @@ test_layers_filter_unload (void)
 }
 
 static int
-test_layers_filter_config (nbdkit_next_config *next, void *nxdata,
+test_layers_filter_config (nbdkit_next_config *next, nbdkit_backend *nxdata,
                            const char *key, const char *value)
 {
   DEBUG_FUNCTION;
@@ -73,7 +72,7 @@ test_layers_filter_config (nbdkit_next_config *next, void *nxdata,
 
 static int
 test_layers_filter_config_complete (nbdkit_next_config_complete *next,
-                                    void *nxdata)
+                                    nbdkit_backend *nxdata)
 {
   DEBUG_FUNCTION;
   return next (nxdata);
@@ -83,53 +82,96 @@ test_layers_filter_config_complete (nbdkit_next_config_complete *next,
   "test_layers_" layer "_config_help"
 
 static int
-test_layers_filter_get_ready (nbdkit_next_get_ready *next,
-                              void *nxdata, int thread_model)
+test_layers_filter_thread_model (void)
 {
   DEBUG_FUNCTION;
-  return next (nxdata);
+  return NBDKIT_THREAD_MODEL_PARALLEL;
 }
 
 static int
-test_layers_filter_after_fork (nbdkit_next_after_fork *next,
-                               void *nxdata)
+test_layers_filter_get_ready (int thread_model)
 {
   DEBUG_FUNCTION;
-  return next (nxdata);
+  return 0;
+}
+
+static int
+test_layers_filter_after_fork (nbdkit_backend *backend)
+{
+  DEBUG_FUNCTION;
+  saved_backend = backend;
+  return 0;
+}
+
+static void
+test_layers_filter_cleanup (nbdkit_backend *backend)
+{
+  assert (backend == saved_backend);
+  DEBUG_FUNCTION;
 }
 
 static int
 test_layers_filter_preconnect (nbdkit_next_preconnect *next,
-                               void *nxdata, int readonly)
+                               nbdkit_backend *nxdata, int readonly)
 {
+  assert (nxdata == saved_backend);
   DEBUG_FUNCTION;
   return next (nxdata, readonly);
 }
 
 static int
-test_layers_filter_list_exports (nbdkit_next_list_exports *next, void *nxdata,
-                                 int readonly, int default_only,
+test_layers_filter_list_exports (nbdkit_next_list_exports *next,
+                                 nbdkit_backend *nxdata,
+                                 int readonly, int is_tls,
                                  struct nbdkit_exports *exports)
 {
+  assert (nxdata == saved_backend);
   DEBUG_FUNCTION;
-  return next (nxdata, readonly, default_only, exports);
+  return next (nxdata, readonly, exports);
+}
+
+static const char *
+test_layers_filter_default_export (nbdkit_next_default_export *next,
+                                   nbdkit_backend *nxdata, int readonly,
+                                   int is_tls)
+{
+  assert (nxdata == saved_backend);
+  DEBUG_FUNCTION;
+  return next (nxdata, readonly);
 }
 
 static void *
-test_layers_filter_open (nbdkit_next_open *next, void *nxdata,
+test_layers_filter_open (nbdkit_next_open *next, nbdkit_context *nxdata,
                          int readonly, const char *exportname, int is_tls)
 {
-  struct handle *h = malloc (sizeof *h);
+  struct handle *h = calloc (1, sizeof *h);
 
+  assert (nbdkit_context_get_backend (nxdata) == saved_backend);
   if (!h) {
     perror ("malloc");
     exit (1);
   }
-  h->nxdata = nxdata;
-  h->next_ops = NULL;
 
-  if (next (nxdata, readonly, exportname) == -1)
+  /* Demonstrate our claim that next() is sugar for open-coding. */
+  if (strcmp (layer, "filter2") == 0) {
+    nbdkit_backend *backend;
+    nbdkit_next *n, *old;
+
+    backend = nbdkit_context_get_backend (nxdata);
+    assert (backend != NULL);
+    n = nbdkit_next_context_open (backend, readonly, exportname, 0);
+    if (n == NULL) {
+      free (h);
+      return NULL;
+    }
+    old = nbdkit_context_set_next (nxdata, n);
+    assert (old == NULL);
+    h->next = n;
+  }
+  else if (next (nxdata, readonly, exportname) == -1) {
+    free (h);
     return NULL;
+  }
 
   /* Debug after recursing, to show opposite order from .close */
   DEBUG_FUNCTION;
@@ -145,228 +187,237 @@ test_layers_filter_close (void *handle)
 }
 
 static int
-test_layers_filter_prepare (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_prepare (nbdkit_next *next,
                             void *handle, int readonly)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == NULL);
-  assert (h->nxdata == nxdata);
-  h->next_ops = next_ops;
+  if (strcmp (layer, "filter2") == 0)
+    assert (h->next == next);
+  else {
+    assert (h->next == NULL);
+    h->next = next;
+  }
   DEBUG_FUNCTION;
   return 0;
 }
 
 static int
-test_layers_filter_finalize (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_finalize (nbdkit_next *next,
                              void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
   return 0;
 }
 
 static int64_t
-test_layers_filter_get_size (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_get_size (nbdkit_next *next,
                              void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->get_size (nxdata);
+  return next->get_size (next);
 }
 
 static int
-test_layers_filter_can_write (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_can_write (nbdkit_next *next,
                               void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_write (nxdata);
+  return next->can_write (next);
 }
 
 static int
-test_layers_filter_can_flush (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_can_flush (nbdkit_next *next,
                               void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_flush (nxdata);
+  return next->can_flush (next);
 }
 
 static int
-test_layers_filter_is_rotational (struct nbdkit_next_ops *next_ops,
-                                  void *nxdata,
+test_layers_filter_is_rotational (nbdkit_next *next,
                                   void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->is_rotational (nxdata);
+  return next->is_rotational (next);
 }
 
 static int
-test_layers_filter_can_trim (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_can_trim (nbdkit_next *next,
                              void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_trim (nxdata);
+  return next->can_trim (next);
 }
 
 static int
-test_layers_filter_can_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_can_zero (nbdkit_next *next,
                              void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_zero (nxdata);
+  return next->can_zero (next);
 }
 
 static int
-test_layers_filter_can_fua (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_can_fast_zero (nbdkit_next *next, void *handle)
+{
+  struct handle *h = handle;
+
+  assert (h->next == next);
+  DEBUG_FUNCTION;
+  return next->can_fast_zero (next);
+}
+
+static int
+test_layers_filter_can_fua (nbdkit_next *next,
                             void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_fua (nxdata);
+  return next->can_fua (next);
 }
 
 static int
-test_layers_filter_can_multi_conn (struct nbdkit_next_ops *next_ops,
-                                   void *nxdata,
+test_layers_filter_can_multi_conn (nbdkit_next *next,
                                    void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_multi_conn (nxdata);
+  return next->can_multi_conn (next);
 }
 
 static int
-test_layers_filter_can_extents (struct nbdkit_next_ops *next_ops,
-                                void *nxdata,
+test_layers_filter_can_extents (nbdkit_next *next,
                                 void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_extents (nxdata);
+  return next->can_extents (next);
 }
 
 static int
-test_layers_filter_can_cache (struct nbdkit_next_ops *next_ops,
-                              void *nxdata,
+test_layers_filter_can_cache (nbdkit_next *next,
                               void *handle)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->can_cache (nxdata);
+  return next->can_cache (next);
 }
 
 static int
-test_layers_filter_pread (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_pread (nbdkit_next *next,
                           void *handle, void *buf,
                           uint32_t count, uint64_t offset,
                           uint32_t flags, int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->pread (nxdata, buf, count, offset, flags, err);
+  return next->pread (next, buf, count, offset, flags, err);
 }
 
 static int
-test_layers_filter_pwrite (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_pwrite (nbdkit_next *next,
                            void *handle,
                            const void *buf, uint32_t count, uint64_t offset,
                            uint32_t flags, int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->pwrite (nxdata, buf, count, offset, flags, err);
+  return next->pwrite (next, buf, count, offset, flags, err);
 }
 
 static int
-test_layers_filter_flush (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_flush (nbdkit_next *next,
                           void *handle,
                           uint32_t flags, int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->flush (nxdata, flags, err);
+  return next->flush (next, flags, err);
 }
 
 static int
-test_layers_filter_trim (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_trim (nbdkit_next *next,
                          void *handle, uint32_t count, uint64_t offset,
                          uint32_t flags, int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->trim (nxdata, count, offset, flags, err);
+  return next->trim (next, count, offset, flags, err);
 }
 
 static int
-test_layers_filter_zero (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_zero (nbdkit_next *next,
                          void *handle, uint32_t count, uint64_t offset,
                          uint32_t flags, int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->zero (nxdata, count, offset, flags, err);
+  return next->zero (next, count, offset, flags, err);
 }
 
 static int
-test_layers_filter_extents (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_extents (nbdkit_next *next,
                             void *handle, uint32_t count, uint64_t offset,
                             uint32_t flags, struct nbdkit_extents *extents,
                             int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->extents (nxdata, count, offset, flags, extents, err);
+  return next->extents (next, count, offset, flags, extents, err);
 }
 
 static int
-test_layers_filter_cache (struct nbdkit_next_ops *next_ops, void *nxdata,
+test_layers_filter_cache (nbdkit_next *next,
                           void *handle, uint32_t count, uint64_t offset,
                           uint32_t flags, int *err)
 {
   struct handle *h = handle;
 
-  assert (h->next_ops == next_ops && h->nxdata == nxdata);
+  assert (h->next == next);
   DEBUG_FUNCTION;
-  return next_ops->cache (nxdata, count, offset, flags, err);
+  return next->cache (next, count, offset, flags, err);
 }
 
 static struct nbdkit_filter filter = {
@@ -376,10 +427,13 @@ static struct nbdkit_filter filter = {
   .config            = test_layers_filter_config,
   .config_complete   = test_layers_filter_config_complete,
   .config_help       = test_layers_filter_config_help,
+  .thread_model      = test_layers_filter_thread_model,
   .get_ready         = test_layers_filter_get_ready,
   .after_fork        = test_layers_filter_after_fork,
+  .cleanup           = test_layers_filter_cleanup,
   .preconnect        = test_layers_filter_preconnect,
   .list_exports      = test_layers_filter_list_exports,
+  .default_export    = test_layers_filter_default_export,
   .open              = test_layers_filter_open,
   .close             = test_layers_filter_close,
   .prepare           = test_layers_filter_prepare,
@@ -390,6 +444,7 @@ static struct nbdkit_filter filter = {
   .is_rotational     = test_layers_filter_is_rotational,
   .can_trim          = test_layers_filter_can_trim,
   .can_zero          = test_layers_filter_can_zero,
+  .can_fast_zero     = test_layers_filter_can_fast_zero,
   .can_fua           = test_layers_filter_can_fua,
   .can_multi_conn    = test_layers_filter_can_multi_conn,
   .can_extents       = test_layers_filter_can_extents,
@@ -403,4 +458,4 @@ static struct nbdkit_filter filter = {
   .cache             = test_layers_filter_cache,
 };
 
-NBDKIT_REGISTER_FILTER(filter)
+NBDKIT_REGISTER_FILTER (filter)
