@@ -43,6 +43,10 @@ requires_filter luks
 # qemu-img: luks-copy-zero1.img: Unsupported cipher mode xts
 requires_not test "$(uname)" = "Darwin"
 
+# It takes several minutes to valgrind the 'gnutls_pbkdf2' function,
+# although it does work.
+skip_if_valgrind
+
 disk=luks-info.img
 info=luks-info.log
 cleanup_fn rm -f $disk $info
