@@ -50,7 +50,7 @@ do_test ()
     # We use jq to normalize the output and convert it to plain text.
     nbdkit \
            null "$1" \
-           --run 'qemu-img map -f raw --output=json $nbd' |
+           --run 'qemu-img map -f raw --output=json "$uri"' |
         jq -c '.[] | {start:.start, length:.length, data:.data, zero:.zero}' \
            > $out
     if ! cmp $out $expected; then
