@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2019-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -36,9 +36,12 @@ set -x
 
 # This is an executable C script using nbdkit-cc-plugin.
 requires_plugin cc
-plugin=$SRCDIR/test-read-password-plugin.c
+plugin=$abs_top_srcdir/tests/test-read-password-plugin.c
 requires test -x $plugin
 
+# This variable is used by $plugin.
+export abs_top_srcdir
+
 # expect on macOS does not work for unclear reasons, skip it.
 requires_not test "$(uname)" = "Darwin"
 
