@@ -46,10 +46,10 @@ touch retry-readonly-count retry-readonly-open-count
 start_t=$SECONDS
 
 # Create a custom plugin which will test retrying.
-nbdkit -v -U - \
+nbdkit -v \
        sh - \
        --filter=retry retry-delay=1 retry-readonly=yes \
-       --run 'qemu-io -f raw -c "w 0 512" -c "w 0 512" $nbd || :' <<'EOF'
+       --run 'qemu-io -f raw -c "w 0 512" -c "w 0 512" "$uri" || :' <<'EOF'
 #!/usr/bin/env bash
 case "$1" in
     open)
