@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2019 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -36,7 +36,8 @@ source ./functions.sh
 set -e
 set -x
 
-requires test `id -u` -eq 0
+requires_root
+requires_caps cap_sys_resource
 requires nbdsh --version
 
 nbdsh -c -<<'EOF'
