@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -43,6 +43,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
+#include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -66,16 +67,19 @@ static int fd = -1;
 static struct stat statbuf;
 static char request[16384];
 static check_request_t check_request;
+static bool head_fails_with_403 = false;
 
 static void *start_web_server (void *arg);
 static void handle_requests (int s);
 static void handle_file_request (int s, enum method method);
 static void handle_mirror_redirect_request (int s);
 static void handle_mirror_data_request (int s, enum method method, char byte);
+static void send_403_forbidden (int s);
 static void send_404_not_found (int s);
 static void send_405_method_not_allowed (int s);
 static void send_500_internal_server_error (int s);
 static void xwrite (int s, const char *buf, size_t len);
+static void xwrite_allow_epipe (int s, const char *buf, size_t len);
 static void xpread (char *buf, size_t count, off_t offset);
 
 static void
@@ -90,14 +94,25 @@ cleanup (void)
   rmdir (tmpdir);
 }
 
+static void
+ignore_sigpipe (void)
+{
+  const struct sigaction sa = { .sa_flags = SA_RESTART, .sa_handler = SIG_IGN };
+  sigaction (SIGPIPE, &sa, NULL);
+}
+
 const char *
-web_server (const char *filename, check_request_t _check_request)
+web_server (const char *filename, check_request_t _check_request,
+            bool _head_fails_with_403)
 {
   struct sockaddr_un addr;
   pthread_t thread;
   int err;
 
+  ignore_sigpipe ();
+
   check_request = _check_request;
+  head_fails_with_403 = _head_fails_with_403;
 
   /* Open the file. */
   fd = open (filename, O_RDONLY|O_CLOEXEC);
@@ -106,13 +121,13 @@ web_server (const char *filename, check_request_t _check_request)
     return NULL;
   }
   if (fstat (fd, &statbuf) == -1) {
-    perror ("stat");
+    perror ("web server: stat");
     goto err1;
   }
 
   /* Create the temporary directory for the socket. */
   if (mkdtemp (tmpdir) == NULL) {
-    perror ("mkdtemp");
+    perror ("web server: mkdtemp");
     goto err1;
   }
 
@@ -120,7 +135,7 @@ web_server (const char *filename, check_request_t _check_request)
   memcpy (sockpath, tmpdir, strlen (tmpdir));
   listen_sock = socket (AF_UNIX, SOCK_STREAM|SOCK_CLOEXEC, 0);
   if (listen_sock == -1) {
-    perror ("socket");
+    perror ("web server: socket");
     goto err2;
   }
 
@@ -132,7 +147,7 @@ web_server (const char *filename, check_request_t _check_request)
   }
 
   if (listen (listen_sock, SOMAXCONN) == -1) {
-    perror ("listen");
+    perror ("web server: listen");
     goto err4;
   }
 
@@ -140,13 +155,13 @@ web_server (const char *filename, check_request_t _check_request)
   err = pthread_create (&thread, NULL, start_web_server, NULL);
   if (err) {
     errno = err;
-    perror ("pthread_create");
+    perror ("web server: pthread_create");
     goto err4;
   }
   err = pthread_detach (thread);
   if (err) {
     errno = err;
-    perror ("pthread_detach");
+    perror ("web server: pthread_detach");
     goto err4;
   }
 
@@ -208,10 +223,14 @@ handle_requests (int s)
       sz = sizeof request - n - 1;
       r = read (s, &request[n], sz);
       if (r == -1) {
-        perror ("read");
-        exit (EXIT_FAILURE);
+        perror ("web server: handle_requests: read");
+        /* This isn't fatal, close the socket and accept a new connection. */
+        goto out_and_close;
       }
       if (r == 0) {
+        /* Client closed the connection after sending it, continue
+         * processing but remember to close after doing so.
+         */
         eof = true;
         break;
       }
@@ -235,27 +254,28 @@ handle_requests (int s)
       n = strcspn (&request[5], " \n\t");
       if (n >= sizeof path) {
         send_500_internal_server_error (s);
-        eof = true;
-        break;
+        goto out_and_close;
       }
       memcpy (path, &request[5], n);
       path[n] = '\0';
+      if (head_fails_with_403) {
+        send_403_forbidden (s);
+        goto out_and_close;
+      }
     }
     else if (strncmp (request, "GET ", 4) == 0) {
       method = GET;
       n = strcspn (&request[4], " \n\t");
       if (n >= sizeof path) {
         send_500_internal_server_error (s);
-        eof = true;
-        break;
+        goto out_and_close;
       }
       memcpy (path, &request[4], n);
       path[n] = '\0';
     }
     else {
       send_405_method_not_allowed (s);
-      eof = true;
-      break;
+      goto out_and_close;
     }
 
     fprintf (stderr, "web server: requested path: %s\n", path);
@@ -275,11 +295,11 @@ handle_requests (int s)
       handle_mirror_data_request (s, method, 2);
     else if (strcmp (path, "/mirror3") == 0) {
       send_404_not_found (s);
-      eof = true;
+      goto out_and_close;
     }
     else if (strncmp (path, "/mirror", 7) == 0) {
       send_500_internal_server_error (s);
-      eof = true;
+      goto out_and_close;
     }
 
     /* Otherwise it's a regular file request.  'path' is ignored, we
@@ -291,6 +311,7 @@ handle_requests (int s)
     fprintf (stderr, "web server: completed request\n");
   }
 
+ out_and_close:
   fprintf (stderr, "web server: closing socket\n");
   close (s);
 }
@@ -346,12 +367,19 @@ handle_file_request (int s, enum method method)
   /* Send the file content. */
   data = malloc (length);
   if (data == NULL) {
-    perror ("malloc");
+    perror ("web server: malloc");
     exit (EXIT_FAILURE);
   }
 
   xpread (data, length, offset);
-  xwrite (s, data, length);
+  if (!head_fails_with_403 || offset != 0)
+    xwrite (s, data, length);
+  else
+    /* In the special case where we are testing the fallback from HEAD
+     * request case, the curl plugin will issue a GET for the whole
+     * data, but not read it all.  Ignore EPIPE errors here.
+     */
+    xwrite_allow_epipe (s, data, length);
 
   free (data);
 }
@@ -429,7 +457,7 @@ handle_mirror_data_request (int s, enum method method, char byte)
   /* Send the file content. */
   data = malloc (length);
   if (data == NULL) {
-    perror ("malloc");
+    perror ("web server: malloc");
     exit (EXIT_FAILURE);
   }
 
@@ -439,6 +467,16 @@ handle_mirror_data_request (int s, enum method method, char byte)
   free (data);
 }
 
+static void
+send_403_forbidden (int s)
+{
+  const char response[] =
+    "HTTP/1.1 403 Forbidden\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+  xwrite (s, response, strlen (response));
+}
+
 static void
 send_404_not_found (int s)
 {
@@ -480,7 +518,25 @@ xwrite (int s, const char *buf, size_t len)
   while (len > 0) {
     r = write (s, buf, len);
     if (r == -1) {
-      perror ("write");
+      perror ("web server: write");
+      exit (EXIT_FAILURE);
+    }
+    buf += r;
+    len -= r;
+  }
+}
+
+static void
+xwrite_allow_epipe (int s, const char *buf, size_t len)
+{
+  ssize_t r;
+
+  while (len > 0) {
+    r = write (s, buf, len);
+    if (r == -1) {
+      if (errno == EPIPE)
+        return;
+      perror ("web server: write");
       exit (EXIT_FAILURE);
     }
     buf += r;
@@ -496,11 +552,11 @@ xpread (char *buf, size_t count, off_t offset)
   while (count > 0) {
     r = pread (fd, buf, count, offset);
     if (r == -1) {
-      perror ("read");
+      perror ("web server: xpread: read");
       exit (EXIT_FAILURE);
     }
     if (r == 0) {
-      fprintf (stderr, "pread: unexpected end of file\n");
+      fprintf (stderr, "web server: pread: unexpected end of file\n");
       exit (EXIT_FAILURE);
     }
     buf += r;
